@@ -6,7 +6,7 @@ Some constraints:
  But we *can* rely on ASDF being present *after* we load it.
 * evaluating this file MUST NOT print anything,
  because we use it in the forward-ref test to check that nothing is printed.
-* We make sure that none of our symbols clash with asdf/driver or asdf,
+* We make sure that none of our symbols clash with uiop or asdf,
  so we may use-package them during testing.
 |#
 
@@ -15,11 +15,13 @@ Some constraints:
   (:export
    #:asym #:acall #:asymval
    #:*test-directory* #:*asdf-directory* #:*build-directory* #:*implementation*
+   #:deftest #:is #:signals
    #:assert-compare #:assert-equal #:assert-pathname-equal #:assert-pathnames-equal
    #:hash-table->alist
    #:load-asdf #:maybe-compile-asdf
    #:load-asdf-lisp #:compile-asdf #:load-asdf-fasl
-   #:compile-load-asdf #:load-asdf-system #:clean-load-asdf-system
+   #:compile-load-asdf #:compile-load-asdf-upgrade
+   #:load-asdf-system #:clean-load-asdf-system
    #:register-directory #:load-test-system
    #:with-test #:test-asdf #:debug-asdf
    #:assert-compare
@@ -27,7 +29,7 @@ Some constraints:
    #:leave-test #:def-test-system
    #:action-name #:in-plan-p
    #:test-source #:test-fasl #:resolve-output #:output-location
-   #:quietly))
+   #:quietly #:join-namestrings))
 
 (in-package :asdf-test)
 
@@ -40,7 +42,7 @@ Some constraints:
   `(;; If you want to trace some stuff while debugging ASDF,
     ;; here's a nice place to say what.
     ;; These string designators will be interned in ASDF after it is loaded.
-    
+
     ;;#+ecl ,@'( :perform :input-files :output-files :compile-file* :compile-file-pathname* :load*)
     ))
 
@@ -86,6 +88,7 @@ Some constraints:
         (when errorp (error "Can't find package ~A" pname)))))
 (defun acall (name &rest args)
   (apply (apply 'asym (if (consp name) name (list name))) args))
+(defun ucall (name &rest args) (apply (asym name :uiop) args))
 (defun asymval (name &optional package)
   (symbol-value (asym name package)))
 (defsetf asymval (name &optional package) (new-value) ;; NB: defun setf won't work on GCL2.6
@@ -105,8 +108,32 @@ Some constraints:
 
 (redirect-outputs) ;; Put everything on standard output, for the sake of scripts
 
-;;; Helpful for debugging
+;;; Poor man's test suite, lacking stefil.
+(defmacro deftest (name formals &body body)
+  `(defun ,name ,formals ,@body))
+(defmacro is (x)
+  `(progn
+     (format *error-output* "~&Checking whether ~S~%" ',x)
+     (finish-output *error-output*)
+     (assert ,x)))
+(defmacro signals (condition sexp &aux (x (gensym)))
+  `(progn
+     (format *error-output* "~&Checking whether ~S signals ~S~%" ',sexp ',condition)
+     (finish-output *error-output*)
+     (handler-case
+         ,sexp
+       (,condition (,x)
+         (format *error-output* "~&Received signal ~S~%" ,x)
+         (finish-output *error-output*)
+         t)
+       (:no-error (&rest ,x)
+         (error "Expression ~S fails to raise condition ~S, instead returning~{ ~S~}"
+                ',sexp ',condition ,x))
+       (t (,x)
+         (error "Expression ~S raises signal ~S, not ~S" ',sexp ,x ',condition)))))
 
+
+;;; Helpful for debugging
 (defun pathname-components (p)
   (when p
     (let ((p (pathname p)))
@@ -121,7 +148,7 @@ Some constraints:
   (cond
     ((equal x y)
      (format t "~S and ~S both evaluate to same path:~%  ~S~%" qx qy x))
-    ((acall :pathname-equal x y)
+    ((ucall :pathname-equal x y)
      (warn "These two expressions yield pathname-equal yet not equal path~%~
         the first expression ~S yields this:~%  ~S~%  ~S~%
         the other expression ~S yields that:~%  ~S~%  ~S~%"
@@ -197,7 +224,7 @@ Some constraints:
 (defun asdf-lisp (&optional tag)
   (make-pathname :name (asdf-name tag) :type "lisp" :defaults *build-directory*))
 (defun debug-lisp ()
-  (make-sub-pathname :directory (relative-dir "contrib") :name "debug" :type "lisp" :defaults *asdf-directory*))
+  (make-sub-pathname :directory (relative-dir "contrib") :name "debug" :type "lisp" :defaults *uiop-directory*))
 (defun early-compile-file-pathname (file)
   (compile-file-pathname
    (make-pathname :name (pathname-name file) :type "lisp" :defaults *early-fasl-directory*)))
@@ -231,14 +258,14 @@ Some constraints:
             (decode-universal-time stamp #+gcl2.6 -5) ;; -5 is for *my* localtime
           (unless in-filesystem
             (error "Y U NO use stamp cache?"))
-          (acall :run-program
+          (ucall :run-program
                  `("touch" "-t" ,(format nil "~4,'0D~2,'0D~2,'0D~2,'0D~2,'0D.~2,'0D"
                                          year month day hr min sec)
-                           ,(acall :native-namestring file)))
+                           ,(ucall :native-namestring file)))
           (assert-equal (file-write-date file) stamp)))))
 (defun mark-file-deleted (file)
   (unless (asymval :*asdf-cache*) (error "Y U NO use asdf cache?"))
-  (acall :register-file-stamp file nil))
+  (acall :register-file-stamp (acall :normalize-namestring file) nil))
 
 (defun hash-table->alist (table)
   (loop :for key :being :the :hash-keys :of table :using (:hash-value value)
@@ -299,7 +326,7 @@ is bound, write a message and exit on an error.  If
                              (break))
                             (t
                              (ignore-errors
-                              (acall :print-condition-backtrace
+                              (ucall :print-condition-backtrace
                                      c :count 69 :stream *error-output*))
                              (leave-test "Script failed" 1))))))
               (funcall (or (asym :call-with-asdf-cache) 'funcall) thunk)
@@ -320,7 +347,7 @@ is bound, write a message and exit on an error.  If
   (verbose t nil)
   (loop :for file :in files :do
     (load (string-downcase file)))
-  (setf *package* (some 'find-package '(:asdf :asdf/driver :asdf/utility :asdf/package :asdf-test)))
+  (setf *package* (some 'find-package '(:asdf :uiop :asdf/utility :asdf/package :asdf-test)))
   (load "contrib/debug.lisp"))
 
 (defun load-asdf-lisp (&optional tag)
@@ -363,7 +390,7 @@ is bound, write a message and exit on an error.  If
 (defmacro with-asdf-conditions ((&optional verbose) &body body)
   `(call-with-asdf-conditions #'(lambda () ,@body) ,verbose))
 
-(defun compile-asdf (&optional tag verbose)
+(defun compile-asdf (&optional tag verbose upgradep)
   (let* ((alisp (asdf-lisp tag))
          (afasl (asdf-fasl tag))
          (tmp (make-pathname :name "asdf-tmp" :defaults afasl)))
@@ -391,6 +418,7 @@ is bound, write a message and exit on an error.  If
             ;; ECL 11.1.1 has spurious warnings, same with XCL 0.0.0.291.
             ;; SCL has no warning but still raises the warningp flag since 2.20.15 (?)
             #+(or clisp cmu ecl scl xcl) (good :expected-style-warnings)
+            (and upgradep (good :unexpected-style-warnings))
             (bad :unexpected-style-warnings)))
           (t (good :success)))))))
 
@@ -428,16 +456,20 @@ is bound, write a message and exit on an error.  If
       (:success
        (leave-test "ASDF compiled cleanly" 0)))))
 
-(defun compile-load-asdf (&optional tag)
+(defun compile-load-asdf (&optional tag upgradep)
   ;; emulate the way asdf upgrades itself: load source, compile, load fasl.
   (load-asdf-lisp tag)
-  (let ((results (compile-asdf tag)))
+  (let ((results (compile-asdf tag nil upgradep)))
     (ecase results
       ((:no-output :unexpected-full-warnings :unexpected-style-warnings)
-       (warn "ASDF compiled with ~S" results)
-       (leave-test "failed to compile ASDF" 1))
-      ((:expected-full-warnings :expected-style-warnings :success)
-       (load-asdf-fasl tag)))))
+         (warn "ASDF compiled with ~S" results)
+         (unless (and upgradep (eq results :unexpected-style-warnings))
+           (leave-test "failed to compile ASDF" 1)))
+      ((:expected-full-warnings :expected-style-warnings :success)))
+    (load-asdf-fasl tag)))
+
+(defun compile-load-asdf-upgrade (&optional tag)
+  (compile-load-asdf tag t))
 
 ;;; Now, functions to compile and load ASDF.
 
@@ -520,7 +552,7 @@ is bound, write a message and exit on an error.  If
 (defun frob-packages ()
   (format t "Frob packages~%")
   (use-package :asdf :asdf-test)
-  (when (find-package :asdf/driver) (use-package :asdf/driver :asdf-test))
+  (when (find-package :uiop) (use-package :uiop :asdf-test))
   (when (find-package :asdf/cache) (use-package :asdf/cache :asdf-test))
   (setf *package* (find-package :asdf-test))
   t)
@@ -576,6 +608,7 @@ is bound, write a message and exit on an error.  If
 (defun test-upgrade (old-method new-method tag) ;; called by run-test
   (with-test ()
     (verbose t nil)
+    (setf tag (string tag))
     (when old-method
       (cond
         ((string-equal tag "REQUIRE")
@@ -601,6 +634,12 @@ is bound, write a message and exit on an error.  If
     (assert (asymval '#:*file1* :test-package))
     (assert (asymval '#:*file3* :test-package))))
 
+(defun join-namestrings (namestrings)
+  (with-output-to-string (s)
+    (loop :with separator = (acall :inter-directory-separator)
+          :for (n . morep) :on namestrings
+          :do (format s "~A~@[~C~]" n (and morep separator)))))
+
 ;; These are shorthands for interactive debugging of test scripts:
 (!a
  common-lisp-user::debug-asdf debug-asdf
