@@ -6,7 +6,7 @@
   (:recycle :uiop/os :asdf/os :asdf)
   (:use :uiop/common-lisp :uiop/package :uiop/utility)
   (:export
-   #:featurep #:os-unix-p #:os-windows-p #:os-genera-p #:detect-os ;; features
+   #:featurep #:os-unix-p #:os-macosx-p #:os-windows-p #:os-genera-p #:detect-os ;; features
    #:getenv #:getenvp ;; environment variables
    #:implementation-identifier ;; implementation identifier
    #:implementation-type #:*implementation-type*
@@ -20,6 +20,11 @@
 ;;; Features
 (with-upgradability ()
   (defun featurep (x &optional (*features* *features*))
+    "Checks whether a feature expression X is true with respect to the *FEATURES* set,
+as per the CLHS standard for #+ and #-. Beware that just like the CLHS,
+we assume symbols from the KEYWORD package are used, but that unless you're using #+/#-
+your reader will not have magically used the KEYWORD package, so you need specify
+keywords explicitly."
     (cond
       ((atom x) (and (member x *features*) t))
       ((eq :not (car x)) (assert (null (cddr x))) (not (featurep (cadr x))))
@@ -28,22 +33,38 @@
       (t (error "Malformed feature specification ~S" x))))
 
   (defun os-unix-p ()
+    "Is the underlying operating system some Unix variant?"
     (or #+abcl (featurep :unix)
         #+(and (not abcl) (or unix cygwin darwin)) t))
 
+  (defun os-macosx-p ()
+    "Is the underlying operating system MacOS X?"
+    ;; OS-MACOSX is not mutually exclusive with OS-UNIX,
+    ;; in fact the former implies the latter.
+    (or
+     #+allegro (featurep :macosx)
+     #+clisp (featurep :macos)
+     (featurep :darwin)))
+
   (defun os-windows-p ()
+    "Is the underlying operating system Microsoft Windows?"
     (or #+abcl (featurep :windows)
         #+(and (not (or abcl unix cygwin darwin)) (or win32 windows mswindows mingw32)) t))
 
   (defun os-genera-p ()
+    "Is the underlying operating system Genera (running on a Symbolics Lisp Machine)?"
     (or #+genera t))
 
   (defun os-oldmac-p ()
+    "Is the underlying operating system an (emulated?) MacOS 9 or earlier?"
     (or #+mcl t))
 
   (defun detect-os ()
+    "Detects the current operating system. Only needs be run at compile-time,
+except on ABCL where it might change between FASL compilation and runtime."
     (loop* :with o
            :for (feature . detect) :in '((:os-unix . os-unix-p) (:os-windows . os-windows-p)
+                                         (:os-macosx . os-macosx-p)
                                          (:genera . os-genera-p) (:os-oldmac . os-oldmac-p))
            :when (and (not o) (funcall detect)) :do (setf o feature) (pushnew o *features*)
            :else :do (setf *features* (remove feature *features*))
@@ -57,6 +78,9 @@ that is neither Unix, nor Windows, nor Genera, nor even old MacOS.~%Now you port
 
 (with-upgradability ()
   (defun getenv (x)
+    "Query the environment, as in C getenv.
+Beware: may return empty string if a variable is present but empty;
+use getenvp to return NIL in such a case."
     (declare (ignorable x))
     #+(or abcl clisp ecl xcl) (ext:getenv x)
     #+allegro (sys:getenv x)
@@ -98,6 +122,7 @@ then returning the non-empty string value of the variable"
 
 (with-upgradability ()
   (defun first-feature (feature-sets)
+    "A helper for various feature detection functions"
     (dolist (x feature-sets)
       (multiple-value-bind (short long feature-expr)
           (if (consp x)
@@ -107,15 +132,18 @@ then returning the non-empty string value of the variable"
           (return (values short long))))))
 
   (defun implementation-type ()
+    "The type of Lisp implementation used, as a short UIOP-standardized keyword"
     (first-feature
      '(:abcl (:acl :allegro) (:ccl :clozure) :clisp (:corman :cormanlisp)
        (:cmu :cmucl :cmu) :ecl :gcl
        (:lwpe :lispworks-personal-edition) (:lw :lispworks)
        :mcl :mkcl :sbcl :scl (:smbx :symbolics) :xcl)))
 
-  (defvar *implementation-type* (implementation-type))
+  (defvar *implementation-type* (implementation-type)
+    "The type of Lisp implementation used, as a short UIOP-standardized keyword")
 
   (defun operating-system ()
+    "The operating system of the current host"
     (first-feature
      '(:cygwin (:win :windows :mswindows :win32 :mingw32) ;; try cygwin first!
        (:linux :linux :linux-target) ;; for GCL at least, must appear before :bsd
@@ -124,6 +152,7 @@ then returning the non-empty string value of the variable"
        :genera)))
 
   (defun architecture ()
+    "The CPU architecture of the current host"
     (first-feature
      '((:x64 :x86-64 :x86_64 :x8664-target :amd64 (:and :word-size=64 :pc386))
        (:x86 :x86 :i386 :i486 :i586 :i686 :pentium3 :pentium4 :pc386 :iapx386 :x8632-target)
@@ -144,6 +173,7 @@ then returning the non-empty string value of the variable"
         (error "Can't determine fasl version.")))
 
   (defun lisp-version-string ()
+    "return a string that identifies the current Lisp implementation version"
     (let ((s (lisp-implementation-version)))
       (car ; as opposed to OR, this idiom prevents some unreachable code warning
        (list
@@ -179,6 +209,8 @@ then returning the non-empty string value of the variable"
         s))))
 
   (defun implementation-identifier ()
+    "Return a string that identifies the ABI of the current implementation,
+suitable for use as a directory name to segregate Lisp FASLs, C dynamic libraries, etc."
     (substitute-if
      #\_ #'(lambda (x) (find x " /:;&^\\|?<>(){}[]$#`'\""))
      (format nil "~(~a~@{~@[-~a~]~}~)"
@@ -192,6 +224,7 @@ then returning the non-empty string value of the variable"
 
 (with-upgradability ()
   (defun hostname ()
+    "return the hostname of the current host"
     ;; Note: untested on RMCL
     #+(or abcl clozure cmu ecl genera lispworks mcl mkcl sbcl scl xcl) (machine-instance)
     #+cormanlisp "localhost" ;; is there a better way? Does it matter?
@@ -205,6 +238,7 @@ then returning the non-empty string value of the variable"
 
   #+cmu
   (defun parse-unix-namestring* (unix-namestring)
+    "variant of LISP::PARSE-UNIX-NAMESTRING that returns a pathname object"
     (multiple-value-bind (host device directory name type version)
         (lisp::parse-unix-namestring unix-namestring 0 (length unix-namestring))
       (make-pathname :host (or host lisp::*unix-host*) :device device
@@ -254,22 +288,27 @@ then returning the non-empty string value of the variable"
 ;;;; Jesse Hager: The Windows Shortcut File Format.
 ;;;; http://www.wotsit.org/list.asp?fc=13
 
-#-(or clisp genera) ; CLISP doesn't need it, and READ-SEQUENCE annoys old Genera.
+#-(or clisp genera) ; CLISP doesn't need it, and READ-SEQUENCE annoys old Genera that doesn't need it
 (with-upgradability ()
   (defparameter *link-initial-dword* 76)
   (defparameter *link-guid* #(1 20 2 0 0 0 0 0 192 0 0 0 0 0 0 70))
 
   (defun read-null-terminated-string (s)
+    "Read a null-terminated string from an octet stream S"
+    ;; note: doesn't play well with UNICODE
     (with-output-to-string (out)
       (loop :for code = (read-byte s)
             :until (zerop code)
             :do (write-char (code-char code) out))))
 
   (defun read-little-endian (s &optional (bytes 4))
+    "Read a number in little-endian format from an byte (octet) stream S,
+the number having BYTES octets (defaulting to 4)."
     (loop :for i :from 0 :below bytes
           :sum (ash (read-byte s) (* 8 i))))
 
   (defun parse-file-location-info (s)
+    "helper to parse-windows-shortcut"
     (let ((start (file-position s))
           (total-length (read-little-endian s))
           (end-of-header (read-little-endian s))
@@ -293,6 +332,8 @@ then returning the non-empty string value of the variable"
                   (read-null-terminated-string s))))))
 
   (defun parse-windows-shortcut (pathname)
+    "From a .lnk windows shortcut, extract the pathname linked to"
+    ;; NB: doesn't do much checking & doesn't look like it will work well with UNICODE.
     (with-open-file (s pathname :element-type '(unsigned-byte 8))
       (handler-case
           (when (and (= (read-little-endian s) *link-initial-dword*)
